require 'json'
require 'uu/os/lang/json_patch'
require 'uu/os/http/http_method'
require 'uu/os/http/http_response'

module UU
  module OS
    module HTTP

      # GET Method.
      class HTTPGet < UU::OS::HTTP::HTTPMethod

        # Creates new instance of HTTPGet
        # @param [UU::OS::HTTP::HTTPClient] client Instance of HTTPClient
        # @param [String] path Request path
        # @private
        def initialize(client, path) # :nodoc:
          super(client, path, :get)
        end

        # Sets custom Accept header.
        # @param [Array<String>] content_type Accepted content-type.
        # @return [HTTPMethod] Self instance for configuration chaining.
        def accept(*content_type)
          headers[:accept] = content_type
          return self
        end

        # Executes method. In case you need to stream response, {#execute_async execute_async} should be used.
        # @param [Hash] parameters Query parameters in form of simple key-value hash (nested structures are
        #    serialized to JSON). These parameters overwrites parameters with same name given directly via
        #    {HTTPMethod#set_parameter set_parameter} or {HTTPMethod#parameters parameters} methods.
        # @return [UU::OS::HTTP::HTTPResponse] Response object.
        def execute(parameters = nil)
          prepare_query(parameters)
          super()
        end

        # Executes method asynchronously. Unlike {#execute execute}, this call is non blocking.
        # @param [Hash] parameters Query parameters in form of simple key-value hash (nested structures are
        #    serialized to JSON). These parameters overwrites parameters with same name given directly via
        #    {HTTPMethod#set_parameter set_parameter} or {HTTPMethod#parameters parameters} methods.
        # @return [UU::OS::HTTP::HTTPResponse] Response object.
        def execute_async(parameters = nil)
          prepare_query(parameters)
          super()
        end

        private

        # Transforms parameters to valid query parameters.
        # @param [Hash] parameters Parameters to be transformed
        # @return [void]
        # @private
        def prepare_query(parameters) # :nodoc:
          return if parameters.nil?
          raise TypeError, "no implicit conversion of #{parameters.class} to Hash" if !parameters.kind_of?Hash
          parameters.each do |key, value|
            if value.kind_of?(Hash) || value.kind_of?(Array)
              set_parameter(key, value.to_json)
            elsif value.kind_of?(Date) || value.kind_of?(Time)
              set_parameter(key, value.to_json[1..-2])
            else
              set_parameter(key, value)
            end
          end
        end

      end

    end
  end
end
