# TODO Return Future from execute async.

require 'uu/os/http/http_headers'
require 'uu/os/http/http_query'

module UU
  module OS
    module HTTP

      # Generic ancestor for HTTP methods.
      class HTTPMethod

        # Actual method type (POST, GET, etc).
        # @return [Symbol] Method type.
        attr_reader :method

        # Method path.
        # @return [String] Path.
        attr_reader :path

        # Custom method headers.
        # @return [UU::OS::HTTP::HTTPHeaders] Custom headers.
        attr_reader :headers

        # Query parameters.
        # @return [UU::OS::HTTP::HTTPQuery] Query parameters.
        attr_reader :parameters

        # Creates new instance of HTTPMethod
        # @param [UU::OS::HTTP::HTTPClient] client Instance of HTTPClient
        # @param [String] path Request path
        # @param [Symbol] method HTTP method type
        # @private
        def initialize(client, path, method) # :nodoc:
          @client = client
          @path = path
          @method = method
          @headers = UU::OS::HTTP::HTTPHeaders.new
          @parameters = UU::OS::HTTP::HTTPQuery.new
        end

        # Sets custom header.
        # @param [Symbol, String] name Header name (case insensitive).
        # @param [Array<String>] value Header value.
        # @return [HTTPMethod] Self instance for configuration chaining.
        def set_header(name, *value)
          headers[name] = value
          return self
        end

        # Sets query parameter.
        # @param [Symbol, String] name Parameter name.
        # @param [String] value Parameter value.
        # @return [HTTPMethod] Self instance for configuration chaining.
        def set_parameter(name, value)
          parameters[name] = value
          return self
        end

        # Executes method sending given payload. In case you need to stream response or upload
        # large data, {#execute_async execute_async} should be used.
        # @param [String, IO] payload Payload to be sent.
        # @return [UU::OS::HTTP::HTTPResponse] Response object.
        # @private
        def execute(payload = nil) # :nodoc:
          @client.request(method, path, parameters, payload, headers)
        end

        # Executes method sending given payload asynchronously. Unlike {#execute execute}, this call is non blocking.
        # @param [Hash, Array, IO, String] payload Payload to be sent.
        # @return [UU::OS::HTTP::HTTPResponse] Response object.
        # @private
        def execute_async(payload = nil) # :nodoc:
          @client.request_async(method, path, parameters, payload, headers)
        end

        # Override inspect to hide internal structures.
        # @private
        def inspect # :nodoc:
          "#<#{self.class.name}:0x#{(self.object_id << 1).to_s(16)}>"
        end

      end

    end
  end
end
